/**
 * @file Archivo sobre los graficos de la visualizacion, mapa
 * @author Elisa Merino Gonzalez
 */



const WIDTH = window.innerWidth -230;
const HEIGHT = window.innerHeight - 190 ;

const FONT = "IBM Plex Mono";

let svg = d3.select("#grafico")
    .append("svg")
    .attr("width",WIDTH)
    .attr("height",HEIGHT-10);


let countryName;

let emissionText;

let energyData = d3.csv("data/renewable-share-energy.csv");
let mapData = d3.json("data/world.geo.json");

Promise.all([energyData,mapData])
.then((data)=>{
    let energyData = data[0];
    let mapData = data[1];
    let features = mapData.features;

    let year = "2021";

    let colors = updateYear(features,energyData,year);

    let colorTotal = colors[0];
    

    let projection = d3.geoMercator()
                    .fitExtent([[0,40],[WIDTH-30,HEIGHT-70]],mapData);
    let geoPath = d3.geoPath(projection);
    
   
    let map = svg
        .append("g");
    map.append("rect")
	.style("fill","white")
	.attr("x","0")
	.attr("y","0")
	.attr("width",WIDTH)
	.attr("height",HEIGHT);
    
    map.selectAll(".country")
        .data(features)
        .enter()
        .append("path")
        .attr("class","country")
        .attr("d",geoPath)
        .style("stroke","black")
        .style("fill",(data)=>{
            
            return colorTotal(data.properties.energyData || 0);
        })
        .on("mouseover",mouseOver);

    svg.append("text")
	.attr("x",WIDTH/2)
	.attr("y",27)
	.attr("text-anchor","middle")
	.style("fill","black")
	.style("font-family",FONT)
	.style("font-weight","bold")
	.style("font-size","20px")
	.text("Porcentajes Por Países");

    countryName = svg.append("text")
        .attr("id","name")
        .attr("x","10")
        .attr("y","20")
        .style("fill","black")
        .style("font-family",FONT)
        .style("font-weight","bold")
        .text("");
    
    emissionText = svg.append("text")
        .attr("id","emission")
        .attr("x","10")
        .attr("y",40)
        .style("fill","black")
        .style("font-family",FONT)
        .text("");

    populationText = svg.append("text")
        .attr("id","population")
        .attr("x","10")
        .attr("y","70")
        .style("fill","black")
        .style("font-family",FONT)
        .text("");

    landText = svg.append("text")
        .attr("id","land")
        .attr("x","10")
        .attr("y","100")
        .style("fill","black")
        .style("font-family",FONT)
        .text("");
        
    
    d3.select("#year")
    .on("input",function(){
        year = d3.select("#year").property("value");
        d3.select("#yearOutput").text(year);

        let colors = updateYear(features,energyData,year);
        colorTotal = colors[0];

        d3.selectAll(".country")
        .style("fill",changeColor);

    });

    d3.select("#yearOutput").text(d3.select("#year").property("value"));
    
    function changeColor(data){

        return colorTotal(data.properties.energyData || 0);
    }


            
        //Scale: gradient scale from white (low %) to blue (hiwh %).
        var defs = svg.append("defs")

        var linearGradient = defs.append("linearGradient")
            .attr("id", "linear-gradient")
            .attr("x1", "0%")
            .attr("y1", "0%")
            .attr("x2", "100%")
            .attr("y2", "0%")

        linearGradient.append("stop")
            .attr("offset", "0%")
            .attr("stop-color", "white")

        linearGradient.append("stop")
            .attr("offset", "100%")
            .attr("stop-color", "blue")

        svg.append("rect")
            .attr("id", "scale")
            .attr("width", 300)
            .attr("height", 20)
            .style("fill", "url(#linear-gradient)")
            .attr("x", 100)
            .attr("y", 500)

        var x = d3.scaleBand().rangeRound([0,300]).padding(.1)
        var xAxis = d3.axisBottom(x)
        x.domain(["0%","25%","50%", "75%"])

        svg.append("g")
            .attr("class", "x axis")
            .attr("transform", "translate(100,520)")
            .call(xAxis);

})
.catch((e)=>{
    alert("Hubo un error al cargar los datos");
    console.error(e);
});


function updateYear(features,energyData,year){

    let max = 0;
    for(let i=0;i<features.length;i++){
        for(let j=0;j<energyData.length;j++){
            if(features[i].properties.iso_a3 === energyData[j].Code &&
                energyData[j].Year == year
                ){
                
                features[i].properties.energyData = parseFloat(energyData[j].Porcentaje);

                if(parseFloat(energyData[j].Porcentaje) > max){
                    max = parseFloat(energyData[j].Porcentaje);
                }
            }
        }
    }
 
    let colorTotal = d3.scalePow()
                .exponent(0.25)
                .domain([0,max])
                .range(['white', 'blue']);
    return [colorTotal];
}

function clone(selector) {
    var node = d3.select(selector).node();
    return d3.select(node.parentNode.appendChild(node.cloneNode(true), node.nextSibling));
}

function mouseOver(data){
    d3.select(".currentCountry").remove();
    let name = data.properties.name;
    let pte = data.properties.energyData;
    countryName.text(name);
    emissionText.text("Total: "+d3.format(".2f")(pte)+" %");
    let country = clone(this);
    country
        .attr("class","currentCountry")
        .style("stroke","red")
        .on("mouseout",function(){
            d3.select(this).remove();
        });
}


